/*
 * shohaku Copyright (C) 2005 tomoya nagatani
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
package shohaku.composer.node.collection;

import java.util.Collection;
import java.util.List;

import shohaku.composer.CompositeException;
import shohaku.composer.Node;
import shohaku.composer.node.AbstractValueNode;
import shohaku.core.collections.GroupUtils;

/**
 * キーと値の要素が一つずつ存在するグループを生成するノードを提供します。
 */
public class SingletonGroupNode extends AbstractValueNode {

    /**
     * 値を生成して返却します。
     * 
     * @return 生成された値
     */
    protected Object generateValue() {
        List args = getNodeContext().getValueElementChildren();
        if (args.isEmpty()) {
            throw new CompositeException("プロパティの値要素が指定されていません。");
        }
        if (args.size() > 2) {
            throw new CompositeException("要素が２個以上の数指定されています。 size:" + args.size());
        }

        Object key = ((Node) args.get(0)).getNodeValue();
        Object value = ((Node) args.get(1)).getNodeValue();
        if (value.getClass().isArray()) {
            return GroupUtils.singletonGroup(key, (Object[]) value);
        } else if (value instanceof Collection) {
            return GroupUtils.singletonGroup(key, (Collection) value);
        } else {
            throw new CompositeException("値の型が不正です、配列またはコレクションで有る必要があります。value type:" + value.getClass());
        }

    }

}
